@@ -18,9 +18,11 @@ import re
 import copy
 
 from . import util_pipe as upipe
+from .util_pipe import get_dataset_name
 from .config_pipe import (dict_user_folders, default_rc_filename,
                           dict_extra_filters, dict_calib_tables, dict_input_folders,
-                          dict_folders, dict_folders_target, )
+                          dict_folders, dict_folders_target,
+                          default_strds, default_nds)
 
 ############################################################
 # Some fixed parameters for the structure
@@ -67,9 +69,9 @@ class InitMuseParameters(object) :
 
         if rc_filename is None :
             if not os.path.isfile(default_rc_filename):
-                upipe.print_warning(("No filename or {default_rc} file "
+                upipe.print_warning(f"No filename or {default_rc_filename} file "
                      "to initialise from. We will use the default hardcoded " 
-                     "in the init_musepipe.py module").format(default_rc=default_rc_filename))
+                     "in the config_pipe.py module")
                 self.init_default_param(dict_user_folders)
 
             else :
@@ -96,10 +98,16 @@ class InitMuseParameters(object) :
 
     def init_default_param(self, dict_param) :
         """Initialise the parameters as defined in the input dictionary
-        Hardcoded in init_musepipe.py
+        Hardcoded in config_pipe.py
+
+        Input
+        -----
+        dict_param: dict
+            Input dictionary defining the attributes
         """
         for key in dict_param:
-            upipe.print_info("Default initialisation of attribute {0}".format(key), pipe=self)
+            upipe.print_info(f"Default initialisation of attribute {key}", 
+                             pipe=self)
             setattr(self, key, dict_param[key])
 
     def read_param_file(self, filename, dict_param) :
@@ -107,10 +115,16 @@ class InitMuseParameters(object) :
         """
         # Testing existence of filename
         if not os.path.isfile(filename) :
-            upipe.print_error(("Input parameter {inputname} cannot be found. "
+            upipe.print_error((f"Input parameter {filename} cannot be found. "
                     "We will use the default hardcoded in the "
-                    "init_musepipe.py module").format(inputname=filename))
-            return
+                    "config_pipe.py module"))
+            # Now relying on the default file but first checking it's there
+            if not os.path.isfile(default_rc_filename):
+                upipe.print_error(f"Default rc file as defined in config_pipe.py "
+                                  f"could not be found ({default_rc_filename}).")
+                return
+            else:
+                filename = default_rc_filename
 
         # If it exists, open and read it
         f_param = open(filename)
@@ -118,8 +132,8 @@ class InitMuseParameters(object) :
 
         # Dummy dictionary to see which items are not initialised
         noninit_dict_param = copy.copy(dict_param)
-        for line in lines :
-            if line[0] in ["#", "%"] : continue 
+        for line in lines:
+            if line[0] in ["#", "%"]: continue
 
             sline = re.split(r'(\s+)', line)
             keyword_name = sline[0]
@@ -130,12 +144,29 @@ class InitMuseParameters(object) :
                 setattr(self, keyword_name, keyword) 
                 # Here we drop the item which was initialised
                 val = noninit_dict_param.pop(keyword_name)
-            else :
+            else:
                 continue
 
         # Listing them as warning and using the hardcoded default
         for key in noninit_dict_param:
             upipe.print_warning(("Parameter {param} not initialised "
                    "We will use the default hardcoded value from "
-                   "init_musepipe.py").format(param=key))
+                   "config_pipe.py").format(param=key))
             setattr(self, key, dict_param[key])
+
+    def _get_dataset_name(self, dataset=None):
+        """Reporting the _get_dataset_name from the InitMuseParam
+        class
+
+        dataset : int
+            dataset number. Default is None.
+        """
+        if dataset is None:
+            if hasattr(self, "dataset"):
+                dataset = self.dataset
+            else:
+                upipe.print_error("No dataset number provided")
+                return "NONAME"
+        return get_dataset_name(dataset, self.strds, self.nds)
+
+
