@@ -19,6 +19,7 @@ from numpy import ma
 import os
 from os.path import join as joinpath
 import glob
+import copy
 
 # Importing mpdaf
 try :
@@ -35,10 +36,14 @@ from mpdaf.drs import PixTable
 from astropy.io import fits as pyfits
 from astropy import units as u
 
+# Scipy erosion
+from scipy import ndimage as ndi
+
 import pymusepipe
 from . import util_pipe as upipe
-from .config_pipe import default_wave_wcs, AO_mask_lambda, dict_extra_filters
-from .util_pipe import filter_list_with_pdict
+from .config_pipe import default_wave_wcs, ao_mask_lambda, dict_extra_filters
+from .util_pipe import (filter_list_with_pdict, filter_list_with_suffix_list,\
+                       add_string, default_strds, default_nds)
 from .cube_convolve import cube_kernel, cube_convolve
 
 def get_sky_spectrum(specname) :
@@ -55,7 +60,7 @@ def get_sky_spectrum(specname) :
     spec = Spectrum(wave=wavein, data=sky['data'], var=sky['stat'])
     return spec
 
-def integrate_spectrum(spectrum, wave_filter, throughput_filter, AO_mask=False):
+def integrate_spectrum(spectrum, wave_filter, throughput_filter, ao_mask=False):
     """Integrate a spectrum using a certain Muse Filter file.
 
     Input
@@ -74,8 +79,8 @@ def integrate_spectrum(spectrum, wave_filter, throughput_filter, AO_mask=False):
     specdata = spectrum.data
 
     # If we have an AO mask, we interpolate the spectrum within that range
-    if AO_mask:
-        outside_AO = (specwave < AO_mask_lambda[0]) | (specwave > AO_mask_lambda[1])
+    if ao_mask:
+        outside_AO = (specwave < ao_mask_lambda[0]) | (specwave > ao_mask_lambda[1])
         good_spectrum = np.interp(specwave, specwave[outside_AO],
                                   specdata[outside_AO])
         # Replacing the interval with interpolated spectrum
@@ -93,12 +98,49 @@ def integrate_spectrum(spectrum, wave_filter, throughput_filter, AO_mask=False):
 
     return flux_cont
 
+class BasicPSF(object):
+    """Basic PSF function and parameters
+    """
+    def __init__(self, function="gaussian", fwhm0=0., nmoffat=2.8, b=0.,
+                 l0=6483.58, psf_array=None):
+
+        if psf_array is not None:
+            self.function = psf_array[0].lower()
+            self.fwhm0 = psf_array[1]
+            self.nmoffat = psf_array[2]
+            self.b = psf_array[3]
+            self.l0 = psf_array[4]
+        else:
+            self.function = function.lower()
+            self.fwhm0 = fwhm0
+            self.nmoffat = nmoffat
+            self.b = b
+            self.l0 = l0
+
+    @property
+    def psf_array(self):
+        return [self.function, self.fwhm0, self.nmoffat, self.l0, self.b]
+
+class BasicFile(object):
+    """Basic file with just the name and some properties
+    to attach to that Cube
+    """
+    def __init__(self, filename, **kwargs):
+        self.filename = filename
+        for key in kwargs:
+            val = kwargs.get(key)
+            setattr(self, key, val)
+
+
 class MuseCubeMosaic(CubeMosaic):
     def __init__(self, ref_wcs, folder_ref_wcs="", folder_cubes="",
                  prefix_cubes="DATACUBE_FINAL_WCS",
                  list_suffix=[], use_fixed_cubes=True,
+                 excluded_suffix=[],
+                 included_suffix=[],
                  prefix_fixed_cubes="tmask", verbose=False,
-                 dict_exposures=None):
+                 dict_exposures=None, dict_psf={},
+                 list_cubes=None):
 
         self.verbose = verbose
         self.folder_cubes = folder_cubes
@@ -106,19 +148,52 @@ class MuseCubeMosaic(CubeMosaic):
         self.ref_wcs = ref_wcs
         if not self._check_folder():
             return
-        full_wcs_name = joinpath(self.folder_ref_wcs, self.ref_wcs)
+        self.full_wcs_name = joinpath(self.folder_ref_wcs, self.ref_wcs)
 
         self.prefix_cubes = prefix_cubes
         self.list_suffix = list_suffix
+        self.excluded_suffix = excluded_suffix
+        self.included_suffix = included_suffix
         self.prefix_fixed_cubes = prefix_fixed_cubes
         self.use_fixed_cubes = use_fixed_cubes
         self.dict_exposures = dict_exposures
+        self.dict_psf = dict_psf
 
         # Building the list of cubes
-        self.build_list()
+        self.build_list(list_cubes)
 
         # Initialise the super class
-        super(MuseCubeMosaic, self).__init__(self.list_cubes, full_wcs_name)
+        super(MuseCubeMosaic, self).__init__(self.cube_names, self.full_wcs_name)
+
+        # Check unit
+        if self.unit == u.dimensionless_unscaled:
+            self._get_unit()
+
+    @property
+    def cube_names(self):
+        return [c.filename for c in self.list_cubes]
+
+    def print_cube_names(self):
+        lnames = copy.copy(self.cube_names)
+        lnames.sort()
+        print("Mosaic Cubes ==========================")
+        for i, name in enumerate(lnames):
+            print(f"#{i+1:02d} - {name}")
+        print("=======================================")
+
+    @property
+    def ncubes(self):
+        return len(self.list_cubes)
+
+    @property
+    def list_cubes(self):
+        if not hasattr(self, "_list_cubes"):
+            self._list_cubes = []
+        return self._list_cubes
+
+    @list_cubes.setter
+    def list_cubes(self, val):
+        self._list_cubes = val
 
     def _check_folder(self):
         if not os.path.isdir(self.folder_cubes):
@@ -131,7 +206,27 @@ class MuseCubeMosaic(CubeMosaic):
             return False
         return True
 
-    def build_list(self, folder_cubes=None, prefix_cubes=None, **kwargs):
+    def _get_unit(self):
+        list_units = []
+        for name in self.cube_names:
+            try:
+                list_units.append(Cube(name).unit)
+            except:
+                pass
+
+        u_units = np.unique(list_units)
+        if len(u_units) == 0:
+            print("Warning: no unit found in input cubes, will set dimensionless one")
+            self.unit = u.dimensionless_unscaled
+        else:
+            self.unit = u_units[0]
+            if len(u_units) != 1:
+                print("Warning: units are not all the same for all input Cubes. "
+                      "  Selecting the first encountered")
+            print(f"Unit of mosaiced Cube = {self.unit.to_string()}")
+
+    def build_list(self, folder_cubes=None, prefix_cubes=None, list_cubes=None,
+                   **kwargs):
         """Building the list of cubes to process
 
         Args:
@@ -151,67 +246,107 @@ class MuseCubeMosaic(CubeMosaic):
         if prefix_cubes is not None:
             self.prefix_cubes = prefix_cubes
 
-        # get the list of cubes and return if 0 found
-        list_existing_cubes = glob.glob("{0}{1}*.fits".format(self.folder_cubes,
-                                                     self.prefix_cubes))
+        if list_cubes == None:
+            # get the list of cubes and return if 0 found
+            list_existing_cubes = glob.glob("{0}{1}*.fits".format(self.folder_cubes,
+                                                         self.prefix_cubes))
 
-        upipe.print_info("Found {} existing Cubes in this folder".format(
-                           len(list_existing_cubes)))
-        # Filter the list with the pointing dictionary if given
-        if self.dict_exposures is not None:
-            upipe.print_info("Will be using a dictionary for "
-                             "selecting the appropriate cubes")
-        list_cubes = filter_list_with_pdict(list_existing_cubes,
-                                            list_pointings=None,
-                                            dict_files=self.dict_exposures)
+            upipe.print_info("Found {} existing Cubes in this folder".format(
+                len(list_existing_cubes)))
 
-        # Take (or not) the fixed Cubes
-        if self.use_fixed_cubes:
-            upipe.print_warning("Using Fixed cubes with prefix {} "
-                                "when relevant".format(self.prefix_fixed_cubes))
-            prefix_to_consider = "{0}{1}".format(self.prefix_fixed_cubes,
-                                                 self.prefix_cubes)
-            list_fixed_cubes = glob.glob("{0}{1}*fits".format(
-                                               self.folder_cubes,
-                                               prefix_to_consider))
-            upipe.print_info("Initial set of {:02d} fixed "
-                             "cubes found".format(len(list_fixed_cubes)))
+            upipe.print_info("Found {} Cubes after suffix filtering".format(
+                len(list_existing_cubes)))
 
-            # Looping over the existing fixed pixtables
-            for fixed_cube in list_fixed_cubes:
-                # Finding the name of the original one
-                orig_cube = fixed_cube.replace(prefix_to_consider,
-                                               self.prefix_cubes)
-                if orig_cube in list_cubes:
-                    # If it exists, remove it
-                    list_cubes.remove(orig_cube)
+            # if the list of exclusion suffix is empty, just use all cubes
+            list_existing_cubes = filter_list_with_suffix_list(list_existing_cubes,
+                                                    self.included_suffix,
+                                                    self.excluded_suffix,
+                                                    name_list="Existing Cubes")
+
+            # Filter the list with the pointing dictionary if given
+            if self.dict_exposures is not None:
+                upipe.print_info("Will be using a dictionary for "
+                                 "further selecting the appropriate cubes")
+            list_cubes = filter_list_with_pdict(list_existing_cubes,
+                                                list_pointings=None,
+                                                dict_files=self.dict_exposures)
+
+            # Take (or not) the fixed Cubes
+            if self.use_fixed_cubes:
+                upipe.print_warning("Using Fixed cubes with prefix {} "
+                                    "when relevant".format(self.prefix_fixed_cubes))
+                prefix_to_consider = "{0}{1}".format(self.prefix_fixed_cubes,
+                                                     self.prefix_cubes)
+                list_fixed_cubes = glob.glob("{0}{1}*fits".format(
+                                                   self.folder_cubes,
+                                                   prefix_to_consider))
+                upipe.print_info("Initial set of {:02d} fixed "
+                                 "cubes found".format(len(list_fixed_cubes)))
+
+                # if the list of exclusion suffix is empty, just use all cubes
+                list_fixed_cubes = filter_list_with_suffix_list(list_fixed_cubes,
+                                                          self.included_suffix,
+                                                          self.excluded_suffix,
+                                                          name_list="Fixed Cubes")
+
+                # Looping over the existing fixed pixtables
+                for fixed_cube in list_fixed_cubes:
+                    # Finding the name of the original one
+                    orig_cube = fixed_cube.replace(prefix_to_consider,
+                                                   self.prefix_cubes)
+                    temp_list = copy.copy(list_cubes)
+                    if orig_cube in temp_list:
+                        # If it exists, remove it
+                        list_cubes.remove(orig_cube)
+                        upipe.print_warning(f"Cube {orig_cube} was removed from "
+                                            f"the list (fixed one will be used)")
+                    else:
+                        upipe.print_warning(f"Original Cube {orig_cube} not "
+                                            f"found (but fixed cube will "
+                                            f"be used nevertheless)")
                     # and add the fixed one
                     list_cubes.append(fixed_cube)
-                    upipe.print_warning("Fixed Cube {} was included "
-                                        "in the list".format(fixed_cube))
-                    upipe.print_warning("and Cube {} was thus removed from "
-                                        "the list".format(orig_cube))
-                else:
-                    upipe.print_warning("Original Cube {} not "
-                                        "found".format(orig_cube))
-                    upipe.print_warning("Hence will not include fixed "
-                                        "Cube in the list {}".format(fixed_cube))
+                    upipe.print_warning(f"Fixed Cube {fixed_cube} has been included "
+                                        f"in the list")
 
-        # if the list of suffix is empty, just use all cubes
-        if len(self.list_suffix) == 0:
-            self.list_cubes = list_cubes
-        else:
-            # Filtering out the ones that don't have any of the suffixes
-            self.list_cubes = []
-            for l in list_cubes:
-                if any([suff in l for suff in self.list_suffix]):
-                    self.list_cubes.append(l)
+            # if the list of suffix is empty, just use all cubes
+            if len(self.list_suffix) > 0:
+                # Filtering out the ones that don't have any of the suffixes
+                temp_list_cubes = []
+                for l in list_cubes:
+                    if any([suff in l for suff in self.list_suffix]):
+                        temp_list_cubes.append(l)
+                list_cubes = temp_list_cubes
+
+        # Attach the other properties to the list of cubes (e.g. PSF)
+        self.list_cubes = []
+        # Names for the OBs
+        strds = kwargs.pop("strds", default_strds)
+        nds = kwargs.pop("nds", default_nds)
+        for name in list_cubes:
+            self.list_cubes.append(BasicFile(name))
+            if len(self.dict_psf) > 0:
+                found = False
+                for key in self.dict_psf:
+                    if found:
+                        break
+                    keyword = f"{self.prefix_cubes}_{get_obname(np.int(key), strds, nds)}"
+                    if keyword in name:
+                        psf = self.dict_psf[key]
+                        self.list_cubes[-1].psf = BasicPSF(psf_array=psf)
+                        found = True
+                # If none correspond, set to the 0 FWHM Gaussian
+                if not found:
+                    upipe.print_warning(f"No PSF found for cube {name}. "
+                                        f"Using default")
+                    self.list_cubes[-1].psf = BasicPSF()
+            else:
+                self.list_cubes[-1].psf = BasicPSF()
 
         if self.verbose:
-            for i, name in enumerate(self.list_cubes):
-                upipe.print_info("Cube {0:03d}: {1}".format(i+1, name))
+            for i, c in enumerate(self.list_cubes):
+                upipe.print_info("Cube {0:03d}: {1}".format(i+1, c.filename))
 
-        self.ncubes = len(self.list_cubes)
         if self.ncubes == 0:
             upipe.print_error("Found 0 cubes in this folder with suffix"
                               " {}: please change suffix".format(
@@ -220,6 +355,42 @@ class MuseCubeMosaic(CubeMosaic):
             upipe.print_info("Found {} cubes to be processed".format(
                                  self.ncubes))
 
+    def convolve_cubes(self, target_fwhm, target_nmoffat=None,
+                        target_function="gaussian", suffix="conv", **kwargs):
+        """
+
+        Args:
+            target_fwhm:
+            target_nmoffat:
+            input_function:
+            target_function:
+            suffix:
+            **kwargs:
+
+        Returns:
+
+        """
+        # Convolving cube per cube
+        for i, c in enumerate(self.list_cubes):
+            # Removing the input folder
+            folder, filename = os.path.split(c.filename)
+            # Getting just the name and the extension
+            name, extension = os.path.splitext(filename)
+            outcube_name = f"{name}_{suffix}{extension}"
+            cube = MuseCube(filename=c.filename, psf_array=c.psf.psf_array)
+            cube_folder, outcube_name = cube.convolve_cube_to_psf(target_fwhm,
+                                      target_nmoffat=target_nmoffat,
+                                      target_function=target_function,
+                                      outcube_name=outcube_name,
+                                      outcube_folder=folder,
+                                      **kwargs)
+
+            # updating the convolved cube name
+            psf = BasicPSF(function=target_function, fwhm0=target_fwhm,
+                           nmoffat=target_nmoffat, l0=c.psf.l0, b=0.)
+            self.list_cubes[i] = BasicFile(joinpath(cube_folder,outcube_name),
+                                           psf=psf)
+
     def madcombine(self, folder_cubes=None, outcube_name="dummy.fits",
                    fakemode=False, mad=True):
         """Combine the CubeMosaic and write it out.
@@ -258,21 +429,18 @@ class MuseCube(Cube):
     def __init__(self, source=None, verbose=False, **kwargs) :
         """Initialisation of the opening of a Cube
         """
+        # PSF for that Cube
+        psf_array = kwargs.pop("psf_array", None)
+        self.psf = BasicPSF(psf_array=psf_array)
+
         if source is not None:
             self.__dict__.update(source.__dict__)
         else :
             Cube.__init__(self, **kwargs)
- 
+
         self.verbose = verbose
         self._debug = kwargs.pop("debug", False)
 
-        # PSF for that Cube
-        self.psf_function = kwargs.pop("psf_function", "moffat")
-        self.psf_fwhm0 = kwargs.pop("psf_fwhm0", 0.5)
-        self.psf_l0 = kwargs.pop("psf_l0", 6483.58)
-        self.psf_nmoffat = kwargs.pop("psf_nmoffat", 2.8)
-        self.psf_b = kwargs.pop("psf_b", -3.0e-5)
-
     def get_spectrum_from_cube(self, nx=None, ny=None, pixel_window=0, title="Spectrum"):
         """Get a spectrum from the cube with centre defined in pixels
         with nx, ny and a window of 'pixel_window'
@@ -332,6 +500,72 @@ class MuseCube(Cube):
         subcube.write(joinpath(cube_folder, outcube_name))
         return cube_folder, outcube_name
 
+    def rebin_spatial(self, factor, mean=False, inplace=False, full_covariance=False, **kwargs):
+        """Combine neighboring pixels to reduce the size of a cube by integer factors along each axis.
+
+        Each output pixel is the mean of n pixels, where n is the product of the 
+        reduction factors in the factor argument.
+        Uses mpdaf rebin function, but add a normalisation factor if mean=False (sum).
+        It also updates the unit by just copying the old one.
+
+        Input
+        -----
+        factor :  (int or (int,int))
+            Factor by which the spatial dimensions are reduced
+        mean : bool
+            If True, taking the mean, if False (default) summing
+        inplace : bool
+            If False (default) making a copy. Otherwise using the present cube.
+        full_covariance: bool
+            If True, will assume that spaxels are fully covariant. This means that
+            the variance will be normalised by sqrt(N) where N is the number of 
+            summed spaxels. Default is False
+
+        Returns
+        -------
+        Cube: rebinned cube
+        """
+        # We copy it to keep the unit :-(
+        res = self if inplace else self.copy()
+
+        # Use the same reduction factor for all dimensions?
+        # Copy from the mpdaf _rebin (in data.py)
+        nfacarr = np.ones((res.ndim), dtype=int)
+        if isinstance(factor, np.int):
+            nfacarr[1:] *= factor
+        elif len(factor) == 2:
+            nfacarr[1:] = np.asarray(factor)
+        elif len(factor) == 3:
+            nfacarr = np.asarray(factor)
+        else:
+            print("Factor should be an integer or list/array of 2 or 3 integers")
+        spafactor = nfacarr[1] * nfacarr[2]
+       
+        # Do the rebin using the rebin method from mpdaf Cube
+        res = res.rebin(nfacarr, inplace=True, **kwargs)
+
+        # Mean or Sum
+        if mean:
+            norm_factor = 1.
+        else:
+            norm_factor = spafactor
+
+        # Now scaling the data and variances
+        res._data *= norm_factor
+        res._var *= norm_factor**2
+
+        # If full covariance, the S/N does not change.
+        # Hence by summing ns spaxels, the S/N should in principle increase by sqrt(ns)
+        # 1- If not mean, it recovers the right factor for the surface
+        #    Namely : data * norm_factor, var * norm_factor**2
+        # 2- If full_covariance, the S/N does not change so we need to multiply the variance
+        #    to compensate. Since the S/N increased by sqrt(ns), we multiply var * ns
+        if full_covariance:
+            print("Warning: Full Covariance option is ON")
+            res._var *= spafactor
+
+        return res
+
     def astropy_convolve(self, other, fft=True, inplace=False):
         """Convolve a DataArray with an array of the same number of dimensions
         using a specified convolution function.
@@ -356,7 +590,7 @@ class MuseCube(Cube):
 
         Parameters
         ----------
-        fn : callable
+        fft : boolean
             The convolution function to use, chosen from:
 
             - `astropy.convolution.convolve_fft'
@@ -423,19 +657,23 @@ class MuseCube(Cube):
         # Calling the external function now
         out._data, out._var = cube_convolve(out._data, kernel,
                                             variance=out._var, fft=fft)
+        # Put back nan in the data and var
+        if masked:
+            out._data[out._mask] = np.nan
+            out._var[out._mask] = np.nan
 
         return out
 
     def convolve_cube_to_psf(self, target_fwhm, target_nmoffat=None,
-                             input_function='moffat', target_function="gaussian",
+                             target_function="gaussian",
+                             outcube_folder=None,
                              outcube_name=None, factor_fwhm=3,
-                             fft=True):
+                             fft=True, erode_edges=True, npixels_erosion=2):
         """Convolve the cube for a target function 'gaussian' or 'moffat'
 
         Args:
             target_fwhm (float): target FWHM in arcsecond
             target_nmoffat: target n if Moffat function
-            input_function (str): 'gaussian' or 'moffat' ['moffat']
             target_function (str): 'gaussian' or 'moffat' ['gaussian']
             factor_fwhm (float): number of FWHM for size of Kernel
             fft (bool): use FFT to convolve or not [False]
@@ -449,42 +687,59 @@ class MuseCube(Cube):
         """
         # Separate folder and name of file
         cube_folder, cube_name = os.path.split(self.filename)
+        if outcube_folder is None:
+            outcube_folder = cube_folder
 
         # Creating the outcube filename
         if outcube_name is None:
             outcube_name = "conv{0}_{1:.2f}{2}".format(target_function.lower()[0],
                                                        target_fwhm, cube_name)
-        upipe.print_info("The new cube will be named: {}".format(outcube_name))
+        else:
+            _, outcube_name = os.path.split(outcube_name)
+        upipe.print_info(f"The new cube will be named: {outcube_name}")
+        upipe.print_info(f"Products will be written in {outcube_folder}")
 
         # Getting the shape of the Kernel
         scale_spaxel = self.get_step(unit_wcs=u.arcsec)[1]
         nspaxel = np.int(factor_fwhm * target_fwhm / scale_spaxel)
         # Make nspaxel odd to have a PSF centred at the centre of the frame
-        if nspaxel % 2 == 0: nspaxel += 1
+        if nspaxel % 2 == 0:
+            nspaxel += 1
         shape = [self.shape[0], nspaxel, nspaxel]
 
         # Computing the kernel
-        kernel3d = cube_kernel(shape, self.wave.coord(), self.psf_fwhm0,
-                               target_fwhm, input_function, target_function,
-                               lambda0=self.psf_l0,
-                               input_nmoffat=self.psf_nmoffat,
-                               target_nmoffat=target_nmoffat, b=self.psf_b,
+        kernel3d = cube_kernel(shape, self.wave.coord(), self.psf.fwhm0,
+                               target_fwhm, self.psf.function, target_function,
+                               lambda0=self.psf.l0,
+                               input_nmoffat=self.psf.nmoffat,
+                               target_nmoffat=target_nmoffat, b=self.psf.b,
                                scale=scale_spaxel, compute_kernel='pypher')
 
         # Calling the local method using astropy convolution
         conv_cube = self.astropy_convolve(other=kernel3d, fft=fft)
+        # Copying the unit from the original to the convolved cube
+        conv_cube.unit = copy.copy(self.unit)
+
+        # Erode by npixels in case erode is True
+        if erode_edges:
+            nmask = ~ndi.binary_erosion(~np.isnan(conv_cube._data), 
+                                        mask=~conv_cube._mask, 
+                                        iterations=npixels_erosion)
+            conv_cube._data[nmask] = np.nan 
+            conv_cube._var[nmask] = np.nan 
+            conv_cube._mask = nmask
 
         # Write the output
         upipe.print_info("Writing up the derived cube")
-        conv_cube.write(joinpath(cube_folder, outcube_name))
+        conv_cube.write(joinpath(outcube_folder, outcube_name))
 
         # Write the kernel3D
         upipe.print_info("Writing up the used kernel")
         kercube = Cube(data=kernel3d)
-        kercube.write(joinpath(cube_folder, "ker3d_{}".format(outcube_name)))
+        kercube.write(joinpath(outcube_folder, "ker3d_{}".format(outcube_name)))
 
         # just provide the output name by folder+name
-        return cube_folder, outcube_name
+        return outcube_folder, outcube_name
 
     def create_reference_cube(self, lambdamin=4700, lambdamax=9400,
             step=1.25, outcube_name=None, filter_for_nan=False, **kwargs):
@@ -591,6 +846,38 @@ class MuseCube(Cube):
         return MuseImage(self.select_lambda(lmin, lmax).sum(axis=0), 
                 title="{0} map".format(line))
 
+    def build_filterlist_images(self, filter_list, prefix="IMAGE_FOV",
+                              suffix="", folder=None, **kwargs):
+        """
+
+        Args:
+            filter_list:
+            prefix:
+            suffix:
+            folder:
+            **kwargs:
+
+        Returns:
+
+        """
+        upipe.print_info("Building images for each filter in given list")
+        cube_folder, cube_name = os.path.split(self.filename)
+        if folder is None:
+            folder = cube_folder
+
+        suffix = add_string(suffix)
+
+        for filtername in filter_list:
+            upipe.print_info(f"Filter = {filtername}")
+            ima = self.get_filter_image(filter_name=filtername, **kwargs)
+            if ima is None:
+                upipe.print_error(f"Could not reconstruct Image with Filter {filtername}")
+                continue
+
+            ima_name = f"{prefix}_{filtername}{suffix}.fits"
+            upipe.print_info(f"Writing image {ima_name}")
+            ima.write(joinpath(folder, ima_name))
+
     def get_filter_image(self, filter_name=None, own_filter_file=None, filter_folder="",
             dict_filters=None):
         """Get an image given by a filter. If the filter belongs to
@@ -629,17 +916,24 @@ class MuseCube(Cube):
                     upipe.print_error("[mpdaf_pipe / get_filter_image] "
                                       "No extra filter dictionary and "
                                       "the private filter file is not set - Aborting")
-                    return
+                    return None
                 else:
                     filter_file = own_filter_file
             # Now reading the filter data
             path_filter = joinpath(filter_folder, filter_file)
             filter_wave, filter_sensitivity = np.loadtxt(path_filter, unpack=True)
             upipe.print_info("Building Image with filter {}".format(filter_name))
+            # using mpdaf bandpass_image to build the image
             refimage = self.bandpass_image(filter_wave, filter_sensitivity,
                                            interpolation='linear')
+
+            # Adding FILTER NAME to the primary header
             key = 'HIERARCH ESO DRS MUSE FILTER NAME'
             refimage.primary_header[key] = (filter_name, 'filter name used')
+
+            # Copying the unit from the cube
+            refimage.unit = copy.copy(self.unit)
+
             add_mpdaf_method_keywords(refimage.primary_header, 
                     "cube.bandpass_image", ['name'], 
                     [filter_name], ['filter name used'])
@@ -699,11 +993,11 @@ class MuseCube(Cube):
 
         # Rewrite a new cube
         if save:
-            suffix_out = kwargs.pop("suffix_out", "tmask")
+            prefix = kwargs.pop("prefix", "tmask")
             use_folder = kwargs.pop("use_folder", True)
 
             cube_folder, cube_name = os.path.split(self.filename)
-            trailcube_name = "{0}{1}".format(suffix_out, cube_name)
+            trailcube_name = "{0}{1}".format(prefix, cube_name)
             if use_folder:
                 trailcube_name = joinpath(cube_folder, trailcube_name)
 
@@ -740,7 +1034,7 @@ class MuseSkyContinuum(object):
         wavein = WaveCoord(cdelt=cdelt, crval=crval, cunit=u.angstrom)
         self.spec = Spectrum(wave=wavein, data=data)
 
-    def integrate(self, muse_filter, AO_mask=False):
+    def integrate(self, muse_filter, ao_mask=False):
         """Integrate a sky continuum spectrum using a certain filter file.
         If the file is a fits file, use it as the MUSE filter list.
         Otherwise use it as an ascii file
@@ -754,7 +1048,7 @@ class MuseSkyContinuum(object):
         muse_filter.flux_cont = integrate_spectrum(self.spec, 
                                                    muse_filter.wave, 
                                                    muse_filter.throughput,
-                                                   AO_mask=AO_mask)
+                                                   ao_mask=ao_mask)
         setattr(self, muse_filter.filter_name, muse_filter)
 
     def get_normfactor(self, background, filter_name="Cousins_R"):
