@@ -13,6 +13,7 @@ import os
 import time
 from os.path import join as joinpath
 import copy
+from collections import OrderedDict
 
 # Numpy
 import numpy as np
@@ -21,16 +22,16 @@ from astropy import constants as const
 from astropy.io import fits as pyfits
 
 # Import package modules
-from .emission_lines import list_emission_lines
-from .emission_lines import full_muse_wavelength_range
-from .config_pipe import default_filter_list
+from .emission_lines import list_emission_lines, full_muse_wavelength_range
+from .config_pipe import (default_filter_list, dict_musemodes,
+                          default_nds, default_strds)
 from . import util_pipe as upipe
 
+# MPDAF
 from mpdaf.obj import Image, Cube
 
-from collections import OrderedDict
 
-############    PRINTING FUNCTIONS #########################
+#  PRINTING FUNCTIONS #
 HEADER = '\033[95m'
 OKBLUE = '\033[94m'
 OKGREEN = '\033[92m'
@@ -125,6 +126,97 @@ def print_error(text, **kwargs):
 
 #-----------  END PRINTING FUNCTIONS -----------------------
 
+def analyse_musemode(musemode, field, delimiter='-'):
+    """Extract the named field from the musemode
+
+    Input
+    -----
+    musemode: str
+        Mode of the MUSE data to be analysed
+    field: str
+        Field to analyse ('ao', 'field', 'lambda_range')
+    delimiter: str
+        Character to delimit the fields to analyse
+
+    Returns
+    -------
+    val: str
+        Value of the field which was analysed (e.g., 'AO' or 'NOAO')
+    """
+    if field not in dict_musemodes:
+        upipe.error(f"Cannot find such a field ({field}) in the dict_musemodes")
+        return ""
+
+    index = dict_musemodes[field]
+    sval = musemode.split(delimiter)
+
+    if len(sval) < index+1:
+        upipe.print_error(f"Error in analyse_musemode. Cannot access field {index} "
+                          f"After splitting the musemode {musemode} = sval")
+        val = ""
+    else:
+        val = musemode.split(delimiter)[index]
+    return val.lower()
+
+def add_string(text, word="_", loc=0):
+    """Adding string at given location
+    Default is underscore for string which are not empty.
+
+    Input
+    ----
+    text (str): input text
+    word (str): input word to be added
+    loc (int): location in 'text'. [Default is 0=start]
+               If None, will be added at the end.
+
+    Returns
+    ------
+    Updated text
+    """
+    if len(text) > 0:
+        if loc is None:
+            text = f"{text}{word}"
+        else:
+            try:
+                if text[loc] != "_":
+                    text = f"{text[:loc]}{word}{text[loc:]}"
+
+            except:
+                print(f"String index [{loc}] out of range [{len(text)}] in add_string")
+
+    return text
+
+def get_dataset_name(dataset=1, str_dataset=default_strds, ndigits=default_nds):
+    """Formatting for the dataset/field names using the number and
+    the number of digits and prefix string
+
+    Input
+    -----
+    dataset: int
+       Dataset (or Field) number
+    str_dataset: str
+        Prefix representing the dataset (or field)
+    ndigits: int
+        Number of digits to be used for formatting
+
+    Returns
+    -------
+    string for the dataset/field name prefix
+    """
+    return f"{str_dataset}{int(dataset):0{int(ndigits)}}"
+
+def lower_rep(text):
+    """Lower the text and return it after removing all underscores
+
+    Args:
+        text (str): text to treat
+
+    Returns:
+        updated text (with removed underscores and lower-cased)
+
+    """
+    return text.replace("_", "").lower()
+
 def lower_allbutfirst_letter(mystring):
     """Lowercase all letters except the first one
     """
@@ -389,21 +481,21 @@ def reconstruct_filter_images(cubename, filter_list=default_filter_list,
                   filter_list, cubename, filter_fits_file)
     os.system(command)
 
-def add_key_pointing_expo(imaname, iexpo, pointing):
-    """Add pointing and expo number to image
+def add_key_dataset_expo(imaname, iexpo, dataset):
+    """Add dataset and expo number to image
 
     Input
     -----
     imaname: str
     iexpo: int
-    pointing: int
+    dataset: int
     """
-    # Writing the pointing and iexpo in the IMAGE_FOV
+    # Writing the dataset and iexpo in the IMAGE_FOV
     this_image = pyfits.open(imaname, mode='update')
-    this_image[0].header['MUSEPIPE_POINTING'] = (pointing, "Pointing number")
+    this_image[0].header['MUSEPIPE_DATASET'] = (dataset, "Dataset number")
     this_image[0].header['MUSEPIPE_IEXPO'] = (iexpo, "Exposure number")
     this_image.flush()
-    print_info("Keywords MUSEPIPE_POINTING/EXPO updated for image {}".format(
+    print_info("Keywords MUSEPIPE_DATASET/EXPO updated for image {}".format(
         imaname))
 
 def rotate_image_wcs(ima_name, ima_folder="", outwcs_folder=None, rotangle=0.,
@@ -480,8 +572,7 @@ def rotate_image_wcs(ima_name, ima_folder="", outwcs_folder=None, rotangle=0.,
             out_name = ima_name.replace(in_suffix, out_suffix)
     else:
         name, extension = os.path.splitext(ima_name)
-        if out_suffix != "":
-            out_suffix = "_{}".format(out_suffix)
+        out_suffix = add_string(out_suffix)
         out_name = "{0}{1}{2}".format(name, out_suffix, extension)
 
     # write output
@@ -573,11 +664,11 @@ def rotate_cube_wcs(cube_name, cube_folder="", outwcs_folder=None, rotangle=0.,
     final_rot_cube.write(joinpath(outwcs_folder, out_name))
     return outwcs_folder, out_name
 
-def filter_list_with_pdict(input_list, list_pointings=None,
+def filter_list_with_pdict(input_list, list_datasets=None,
                            dict_files=None,
                            verbose=True):
     """Filter out exposures (pixtab or cube namelist) using a dictionary which
-    has a list of pointings and for each pointing a list of exposure number.
+    has a list of datasets and for each dataset a list of exposure number.
 
     Args:
         input_list (list of str):  input list to filter
@@ -596,18 +687,18 @@ def filter_list_with_pdict(input_list, list_pointings=None,
         selected_list = []
         # this is the list of exposures to consider
 
-        if list_pointings is None:
-            list_pointings = dict_files.keys()
-        elif not isinstance(list_pointings, list):
-            upipe.print_error("Cannot recognise input pointing(s)")
+        if list_datasets is None:
+            list_datasets = dict_files.keys()
+        elif not isinstance(list_datasets, list):
+            upipe.print_error("Cannot recognise input dataset(s)")
             return selected_list
 
-        for pointing in list_pointings:
-            if pointing not in dict_files:
-                upipe.print_warning("Pointing {} not in dictionary "
-                                    "- skipping".format(pointing))
+        for dataset in list_datasets:
+            if dataset not in dict_files:
+                upipe.print_warning("Dataset {} not in dictionary "
+                                    "- skipping".format(dataset))
             else:
-                list_expo = dict_files[pointing]
+                list_expo = dict_files[dataset]
                 # We loop on that list
                 for expotuple in list_expo:
                     tpl, nexpo = expotuple[0], expotuple[1]
@@ -624,8 +715,47 @@ def filter_list_with_pdict(input_list, list_pointings=None,
                                 break
 
     if verbose:
-        upipe.print_info("Pointings {0} - Selected {1}/{2} files after "
-                         "dictionary filtering".format(list_pointings,
+        upipe.print_info("Datasets {0} - Selected {1}/{2} files after "
+                         "dictionary filtering".format(list_datasets,
                                                 len(selected_list),
                                                 nfiles_input_list))
     return selected_list
+
+def filter_list_with_suffix_list(list_names, included_suffix_list=[],
+                                 excluded_suffix_list=[], name_list=""):
+    """
+
+    Args:
+        list_names (list of str):
+        included_suffix_list (list of str):
+        excluded_suffix_list (list of str):
+
+    Returns:
+
+    """
+    if name_list is not None:
+        add_message = f"for list {name_list}"
+    else:
+        add_message = ""
+
+    # if the list of inclusion suffix is empty, just use all cubes
+    if len(included_suffix_list) > 0:
+        upipe.print_info(f"Using suffixes {included_suffix_list} "
+                         f"as an inclusive condition {add_message}")
+        # Filtering out the ones that don't have any of the suffixes
+        temp_list = copy.copy(list_names)
+        for l in temp_list:
+            if any([suff not in l for suff in included_suffix_list]):
+                _ = list_names.remove(l)
+
+    # if the list of exclusion suffix is empty, just use all cubes
+    if len(excluded_suffix_list) > 0:
+        upipe.print_info(f"Using suffixes {excluded_suffix_list} "
+                         f"as an exclusive condition {add_message}")
+        # Filtering out the ones that have any of the suffixes
+        temp_list = copy.copy(list_names)
+        for l in temp_list:
+            if any([suff in l for suff in excluded_suffix_list]):
+                _ = list_names.remove(l)
+
+    return list_names
